@@ -14,20 +14,25 @@
   exedir=.BAM_PATH
   if(is.null(exedir)){
     ok=FALSE
+    RBaMdir=find.package("RBaM")[1]
     mess=paste0('Path to BaM executable has not been defined yet.',
-                '\nIf BaM executable is already installed on your computer, use ',
-                'setPathToBaM("folder/containing/BaM/on/your/computer") ',
+                '\n\nIf BaM executable is already installed on your computer, use ',
+                'setPathToBaM(folder_containing_BaM_on_your_computer) ',
                 'to define its location.',
-                '\nOtherwise use downloadBaM("destination/folder/on/your/computer") ',
-                'to download it now.')
+                '\n\nOtherwise use downloadBaM(destination_folder_on_your_computer) ',
+                'to download it now.',
+                '\nFor instance, to download it into RBaM installation folder, type:',
+                '\ndownloadBaM("',file.path(RBaMdir,'bin'),'")')
   } else {
     ok=foundBaM(exedir=exedir,exename='BaM')
     mess=paste0('BaM executable was not found in the folder: ',exedir,
-                '\nTo update BaM location, use setPathToBaM("folder/containing/BaM/on/your/computer").',
+                '\nTo update BaM location, use setPathToBaM(folder_containing_BaM_on_your_computer).',
                 '\nTo re-download BaM executable in this folder, use downloadBaM("',exedir,'")')
   }
   if(!ok){
+    packageStartupMessage('~~~~~~~~~~~~~~~~')
     packageStartupMessage('Welcome to RBaM!')
+    packageStartupMessage('~~~~~~~~~~~~~~~~')
     packageStartupMessage(mess)
   }
 }
